//=========================================================================
//
//	FILE:				DragDrop_App.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_DragDrop demo app.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 4 $
//						Latest Checkout:  $Date: 2005-09-09 22:34:33 -0700 (Fri, 09 Sep 2005) $
//
//=========================================================================

	// CP_Scrolling
#include "DragDrop_App.h"
#include "DragDrop_ContentsView.h"
#include "DragDrop_DragSourceView.h"

	// CPLAT_Framework
#include "CP_Application.tmpl.h"
#include "CP_DocumentWindow.h"
#include "CP_Exceptions.h"
#include "CP_MenuBarManager.h"
#include "CP_New.h"
#include "CP_Scroller.h"
#include "CP_Types.h"
#include "CP_Utility.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DragDrop_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
DragDrop_Application::DragDrop_Application()
{
		// install the default application command handler callbacks
	CPLAT::CP_CommandHandlerSignal			*sourceWindowSignal = CP_new CPLAT::CP_CommandHandlerSignal();
	sourceWindowSignal->connect( CPLAT::CP_SigSlot::slot(this, &DragDrop_Application::HandleSourceWindow) );
	RegisterCommand( 1000, sourceWindowSignal, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
}

//=========================================================================
//	FUNCTION:	DragDrop_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
DragDrop_Application::~DragDrop_Application() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String DragDrop_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to demonstrate drag and drop.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Commands ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleOpenApplication
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the application is starting up. We take this opportunity to
*	create our initial window and views. In this demo, we create a single window
*	to display the results of dropping items onto it.
*	
*	@exception	none
*
*	@return		void
*/
void DragDrop_Application::HandleOpenApplication()
{
	try
	{
		CPLAT::CP_Window				*window = NULL;
		CPLAT::CP_Point					windowLocation( 10, 50 );
		CPLAT::CP_Size					windowSize( 700, 400 );
		CPLAT::CP_String				windowTitle( CP_T("Drag and Drop Test") );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		
		window = CP_new CPLAT::CP_Window();
		window->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( window );
		
		CPLAT::CP_Point					viewLocation( 0, 0 );
		CPLAT::CP_Size					viewSize( 700, 400 );
		DragDropContentsView			*dragDropContentsView = NULL;
		CPLAT::CP_Scroller				*scroller = NULL;
		
		scroller = CP_new CPLAT::CP_Scroller( window, viewLocation, viewSize );
		
		viewLocation.Set( 1, 1 );
		viewSize.Set( 700 - 18, 400 - 18 );
		dragDropContentsView = CP_new DragDropContentsView( scroller, viewLocation, viewSize );

		scroller->SetScrollingView( dragDropContentsView );
		dragDropContentsView->SetScrollUnit( CPLAT::CP_Size(20, 20) );

		window->Show();
	}
	catch(...)
	{
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Tests ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleSourceWindow
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Creates the drag source window to show how to handle drag source handling
*	when drag and dropping from 1 CPLAT object to another.
*
*	@param		inOutStatus				Not used.
*	@param		inCommandNo				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void DragDrop_Application::HandleSourceWindow( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	try
	{
		CPLAT::CP_Point					windowLocation( 50, 50 );
		CPLAT::CP_Size					windowSize( 200, 200 );
		CPLAT::CP_String				windowTitle( CP_T("Drag Source Window") );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		CPLAT::CP_Window				*dragSourceWindow = NULL;
		
		dragSourceWindow = CP_new CPLAT::CP_Window();
		dragSourceWindow->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_Point					viewLocation( 50, 50 );
		CPLAT::CP_Size					viewSize( 50, 50 );
		DragSourceView					*dragSourceView = NULL;
		
		dragSourceView = CP_new DragSourceView( dragSourceWindow, viewLocation, viewSize );
		
		dragSourceWindow->Show();
	}
	catch(...)
	{
	}
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32
