//=========================================================================
//
//	FILE:				FTP_App.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_FTP demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:42 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================
#ifndef _H_FTP_Application
#define _H_FTP_Application
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Application.h"
#include "CP_DataSource_Strings.h"
#include "CP_DocumentPolicy_Multi.h"
#include "CP_FTP.h"
#include "CP_OSStatus.h"
#include "CP_Socket.h"
#include "CP_PrintPolicy_None.h"
#include "CP_STLArrayDefines.h"
#include "CP_Types.h"
#include "CP_UndoPolicy_None.h"

//!  CPLAT::FTP_Application
/*!
  Application class for the CPLAT::Internet demo. 
*/
class FTP_Application : public CPLAT::CP_Application<CPLAT::CP_DocumentPolicy_Multi, CPLAT::CP_PrintPolicy_None,
	CPLAT::CP_UndoPolicy_None>
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										FTP_Application();
	//! Destructor.
	virtual								~FTP_Application() throw();
	//@}
			
protected:
	//! @name AboutBox
	//@{
	virtual	CPLAT::CP_String			GetAboutBoxDescription() const;
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleOpenApplication();
	//@}

private:
	typedef CPLAT::CP_Application<CPLAT::CP_DocumentPolicy_Multi, CPLAT::CP_PrintPolicy_None, CPLAT::CP_UndoPolicy_None>	inherited;

	//! @name Commands
	//@{
										/*!
										*	Handles the menu/command update status handler for the CP_CMD_New command. We
										*	always enable the command, so many windows can be created if desired.
										*	
										*	@param		inOutCommandStatuss		Loccation to return the enabled status result in.
										*	@param		inOutWasHandled			Not used.
										*	@param		inCommandNo				Not used.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						HandleNewUpdateCommandStatusCommand(
												CPLAT::CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CPLAT::CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( true ); }
	virtual	void 						HandleNewCommand(
												CPLAT::OSStatus& inOutStatus,
												CPLAT::CP_CommandNo& inCommandNo );
	//@}

	//! @name Main Window
	//@{
			void						Connect(
												const CPLAT::CP_String& inHostURL,
												const CPLAT::CP_String& inUser,
												const CPLAT::CP_String& inPassword );

			void						MainWindow();
			void						UpdateDisplayedDirectoryLabel( const CPLAT::CP_String& inDirectory );

			void						GetSiteDialog();

			void						UpdateDirectoryListing();
			bool						ChangeDirectory( const CPLAT::CP_String& inDirectory );
	//@}

	//! @name Slots
	//@{
			void						HandleDismiss(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );
			void						HandleCancel(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );

			void						RootDirectoryClicked(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );
			void						UpDirectoryClicked(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& inOutStatus );

			void						UpdateProgress(
												CPLAT::CP_DataValueHolder& inDataValue,
												CPLAT::OSStatus& inOutStatus );

			void						DirectoryListingDoubleClick(
												CPLAT::CP_DataValueHolder& inValue,
												CPLAT::OSStatus& /*inOutStatus*/ );
	//@}

	// class data
protected:
	CPLAT::CP_Window					*fFTPDisplayWindow;
	
	CPLAT::CP_Socket					*fFTPSocket;
	CPLAT::CP_FTP						*fFTPProtocol;
	
	CPLAT::CP_Strings_Array				fDirectoryListingData;
	
	CPLAT::CP_DataSource_Strings		*fDataSource;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_FTP_Application

