//=========================================================================
//
//	FILE:				Printing_App.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_Printing demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:34:48 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_Printing_Application
#define _H_Printing_Application
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// Printing
#include "Printing_View.h"

	// CPLAT_Framework
#include "CP_Application.h"
#include "CP_DocumentPolicy_None.h"
#include "CP_OSStatus.h"
#include "CP_PrintPolicy_Print.h"
#include "CP_Types.h"
#include "CP_UndoPolicy_None.h"

/*!
*	@class		Printing_Application
*	@brief		Application class for the printing demo.
*/
class Printing_Application : public CPLAT::CP_Application<CPLAT::CP_DocumentPolicy_None, CPLAT::CP_PrintPolicy_Print, CPLAT::CP_UndoPolicy_None>
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										Printing_Application();
	//! Destructor.
	virtual								~Printing_Application() throw();
	//@}

protected:
	//! @name AboutBox
	//@{
	virtual	CPLAT::CP_String			GetAboutBoxDescription() const;
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleOpenApplication();
	//@}

	//! @name Printing
	//@{
	virtual	void						HandlePrinting( CPLAT::CP_Printer *inPrinter );
	//@}

private:
	typedef CPLAT::CP_Application<CPLAT::CP_DocumentPolicy_None, CPLAT::CP_PrintPolicy_Print, CPLAT::CP_UndoPolicy_None>	inherited;

	// class data
protected:
	CPLAT::CP_Window					*fPrintingWindow;		/* ! Window holding the printing views. */
	Printing_View						*fPrintingView1;		/* ! First printing view (top left). */
	Printing_View						*fPrintingView2;		/* ! Second printing view (center). */
	Printing_View						*fPrintingView3;		/* ! First printing view (bottom right). */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_Printing_Application

