//=========================================================================
//
//	FILE:				Threads_RectsView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to display various size rects in various
//						colors in a window.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:33:54 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// HelloWorld
#include "Threads_RectsView.h"
#include "Threads_RectsView.h"

	// CPLAT_Framework
#include "CP_Color.h"
#include "CP_StColorStateSaver.h"
#include "CP_Enums.h"
#include "CP_Graphics.h"
#include "CP_RandomNumbers.h"
#include "CP_SignalSlot.h"
#include "CP_View_Imp_Control.h"
#include "CP_Window.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Threads_RectsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
Threads_RectsView::Threads_RectsView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
	CPLAT::CP_Window		*containingWindow = NULL;
	
	containingWindow = GetContainingWindow();
	
	if( containingWindow != NULL )
		containingWindow->FindSignal( kCP_Signal_WindowClosing )->connect( CPLAT::CP_SigSlot::slot(this, &Threads_RectsView::HandleWindowClosing) );
	
	Resume();		// start the thread running
}

//=========================================================================
//	FUNCTION:	~Threads_RectsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
Threads_RectsView::~Threads_RectsView() throw()
{
	Kill();
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we use the CP_Graphics class to display the line "Hello World from CPLAT"
*	in the default font for a view.
*
*	@param		inCanvas			Not used.
*	@param		inUpdateArea		Not used.
*	
*	@exception	none
*
*	@return		void
*/
void Threads_RectsView::OnDraw( CPLAT::CP_Canvas& /*inCanvas*/, CPLAT::CP_Region& /*inUpdateArea*/ )
{
}

//=========================================================================
//	FUNCTION:	Run
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Draws the rects in the main window.
*
*	@param		UInt32					Result of the thread operation.
*	
*	@exception	none
*
*	@return		void
*/
UInt32 Threads_RectsView::Run()
{
	while( ! HasCompleted() )
	{
		CPLAT::CP_Canvas				tempCanvas = SetupForDrawing();
		
		CPLAT::CP_Graphics				graphics( tempCanvas );
		CPLAT::CP_Rect					viewBounds = GetLocalBounds();

		if( (fIter >= 50) || (fNeedsErasing == true) ) 
		{
			if( fNeedsErasing == true )
				fNeedsErasing = false;
				
				// clear the pane every so often
			CPLAT::CP_StColorStateSaver		colorState( tempCanvas );
			colorState.Default();
			
			graphics.EraseRect( viewBounds );
			
			fIter = 0;
		}
		
		++fIter;
		
		SInt32							startHor = CPLAT::CP_RangedRandom( 0, (SInt16) viewBounds.Width() );
		SInt32							startVer = CPLAT::CP_RangedRandom( 0, (SInt16) viewBounds.Height() );
		SInt32							width = CPLAT::CP_RangedRandom( startHor, (SInt16) viewBounds.Width() - startHor );
		SInt32							height = CPLAT::CP_RangedRandom( startVer, (SInt16) viewBounds.Height() - startVer );
		CPLAT::CP_Rect					shapeBounds( startHor, startVer, startHor + width, startVer + height );
		SInt16							colorIndex = CPLAT::CP_RangedRandom( 1, CPLAT::CP_Color::GetCount() );
		CPLAT::CP_RGBColor				foreColor = CPLAT::CP_Color::GetColor( colorIndex ).GetRGBColor();
		CPLAT::CP_ARGBColor				strokecolor = foreColor;
		
		graphics.SetStrokeColor( strokecolor );
		graphics.SetFillColor( strokecolor );
		graphics.FillRect( shapeBounds );
			
			// frame the rect in a different color
		SInt16							borderWidth = CPLAT::CP_RangedRandom( 1, width / 3 );
		SInt16							borderColorIndex = CPLAT::CP_RangedRandom( 1, CPLAT::CP_Color::GetCount() );
		CPLAT::CP_RGBColor				borderColor = CPLAT::CP_Color::GetColor( borderColorIndex ).GetRGBColor();
		CPLAT::CP_ARGBColor				borderAColor = borderColor;
		
		graphics.SetLineWidth( borderWidth );
		graphics.SetStrokeColor( borderAColor );
		graphics.StrokeRect( shapeBounds );

		GetViewImpBase()->Draw();
		
		CP_M_Thread::YieldThread();

		#if TARGET_OS_MAC
			if( GetContainingWindow() != NULL )
				GetContainingWindow()->Flush();
		#endif
		
		FinishDrawing( tempCanvas );
	}
	
	return( 0 );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleWindowClosing
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the window closes. We use this opportunity to ensure that
*	the thread is killed.
*
*	@param		inValue					Not used.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void Threads_RectsView::HandleWindowClosing( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
		// window is closing, so stop tread
	Kill();
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
