//=========================================================================
//
//	FILE:				CP_AboutBox.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle display of an AboutBox in CPLAT
//						applications.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 267 $
//						Latest Checkout:  $Date: 2005-08-31 19:33:20 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_AboutBox
#define _H_CP_AboutBox
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_M_DeferredTask.h"
#include "CP_OSStatus.h"
#include "CP_SignalSlot.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Application_Imp;

/*!
*	@class		CP_AboutBox
*	@brief		A class to handle display of an AboutBox in CPLAT applications.
*/
class CP_EXPORT CP_AboutBox : public CP_M_DeferredTask, public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AboutBox( CP_Application_Imp *inOwnerApp );
	//! Destructor.
	virtual								~CP_AboutBox() throw();
	//@}
			
	//! @name Instance
	//@{
	static	CP_AboutBox					*Instance();
	//@}

	//! @name Display
	//@{
			void						Display( const CP_String& inDescription = CP_String("") );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the application object that we are displaying the about box for.
										*	
										*	@param		inOwnerApp			The app to display the about box for.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetOwnerApp( CP_Application_Imp *inOwnerApp )		{ fOwnerApp = inOwnerApp; }
	//@}

protected:

private:
	//! @name Task
	//@{
			void						HandleDefaualtButton_Slot(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOutStatus );
	//@}

	// class data
private:
	static CP_AboutBox					*fsInstance;			/*! The single instance of this class. */

protected:
	CP_Application_Imp					*fOwnerApp;				/*! Owner application class. */
	
	CP_Window							*fAboutBoxWindow;		/*! Window displaying the about box. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Display
	//@{
			void						DisplayDefault( const CP_String& inDescription = CP_String("") );

			void						Display(
												const CP_String& inNibName,
												const CP_String& inWindowName,
												UInt32 inDefaultButtonID );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Display
	//@{
			void						DisplayDefault( const CP_String& inDescription = CP_String("") );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_AboutBox

