//=========================================================================
//
//	FILE:				CP_DT_Assertions.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions of various memmory recording assertions.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_DT_Assertions
#define _H_CP_DT_Assertions
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_Types.h"

#define CP_DT_HandleIsPurged(h)					( CP_DT_IS_NULL( *(h) ) )

#define kHStatePurgeableMask	0x40
#define kHStateLockedMask		0x80
#define kHStateResourceMask		0x20

#define CP_DT_HStateIsPurgeable( hState )		( ((hState) & kHStatePurgeableMask) != 0 )
#define CP_DT_HStateIsLocked( hState )			( ((hState) & kHStateLockedMask) != 0 )
#define CP_DT_HStateIsResource( hState )		( ((hState) & kHStateResourceMask) != 0 )

#if CP_Debug_RecordOSCalls
	#define CP_DT_MaybeUnusedArg(x)				{}
#else
		// eliminate "unused argument" complaints from CodeWarrior
	#define CP_DT_MaybeUnusedArg(x)				{ (x) = (x); }
#endif

#define kHStatePurgeableMask	0x40
#define kHStateLockedMask		0x80

void	CP_DT_DebugMsg_Func( char *inMessage );

//==========================================================================
//	Bassic Assertions 
//==========================================================================
#if CP_Debug

	#define CP_DT_DebugMsg( failStr )			{ CP_DT_DebugMsg_Func( failStr ); }
	#define CP_DT_DebugMsgIf( cond, failStr )	{ if( ( cond ) ) { CP_DT_DebugMsg( failStr ); } }
	#define	CP_DT_DebugNum( failStr, num )		{ char msg[256]; CPLAT::CP_DT_GetDebugNumString( failStr, num, msg ); CP_DT_DebugMsg_Func( msg ); }
										
	#define CP_DT_IsValidObject( object )		CP_DT_IsValidObject( object )
	
#else	//	 CP_Debug

	#define CP_DT_DebugMsg( failStr )			{}
	#define CP_DT_DebugMsgIf( cond, failStr )	{}
	#define CP_DT_DebugNum( msg, num )			{}

	#define CP_DT_IsValidObject( object )		{}
	
#endif	//  CP_Debug

//==========================================================================
//	More Complex Assertions 
//==========================================================================
#if CP_Debug

	void	CP_DT_AssertHandleIsValid_Func(
						const void *theHandle,
						const char *cString,
						const char *optionalMsg );
	void	CP_DT_AssertAddressIsValidAlign_Func(
						const void *address,
						const char *cString,
						SInt32 align,
						const char *optionalMsg );

	#define CP_DT_AssertHandleIsValid( h, msg )			CP_DT_AssertHandleIsValid_Func( (h), #h, msg )
	#define CP_DT_AssertResourceIsValid(r, msg )		CP_DT_AssertHandleIsValid_Func( (r), #r, msg )
	#define CP_DT_AssertAddressIsValidAlign(p, a, msg )	CP_DT_AssertAddressIsValidAlign_Func( (p), #p, a, msg )
	#define CP_DT_AssertAddressIsValid(p, msg )			CP_DT_AssertAddressIsValidAlign( p, 1, msg )
	#define CP_DT_AssertAddressIsValidAlign2( p, msg )	CP_DT_AssertAddressIsValidAlign( p, 2, msg )
	#define CP_DT_AssertAddressIsValidAlign4( p, msg )	CP_DT_AssertAddressIsValidAlign( p, 4, msg )

#else	// CP_Debug

	#define CP_DT_AssertHandleIsValid( h, msg )			{}		// empty procedures
	#define CP_DT_AssertResourceIsValid( r, msg )		{}
	#define CP_DT_AssertAddressIsValid( p, msg )		{}
	#define CP_DT_AssertAddressIsValidAlign( p, a, msg ){}
	#define CP_DT_AssertAddressIsValidAlign2( p, msg )	{}
	#define CP_DT_AssertAddressIsValidAlign4( p, msg )	{}
	#define CP_DT_AssertNoErr( err, msg )				{}

#endif	// CP_Debug

//==========================================================================
//	Utility macros 
//==========================================================================
#define CP_DT_IS_NOT_NULL( p )	( (p) != nil )
#define CP_DT_IS_NULL( p )		( (p) == nil )

#define CP_DT_IS_NOT_ERR( e )	( (e) == noErr )
#define CP_DT_IS_ERR( e )		( (e) != noErr )

void	CP_DT_AssertNoErr_Func(
				OSErr err,
				const char *optionalMsg );
void	CP_DT_AssertMemErrIsNormal_Func(
				const char *msg );
void	CP_DT_AssertResErrIsNormal_Func(
				const char *msg );
void 	CP_DT_AssertValidMemoryAllocationSize_Func(
				Size byteCount,
				const char *msg );

bool	CP_DT_MemErrIsNormal_Func(
				OSErr err );
bool	CP_DT_ResErrIsNormal_Func(
				OSErr err );
bool 	CP_DT_IsValidMemoryAllocationSize_Func(
				Size theSize );
void	CP_DT_MemoryDummy_Func(
				SInt16 dummy );

bool	CP_DT_IsValidObject_Func(
				void *object );

#endif		// __BUILD_MAC__

#endif	// _H_CP_DT_Assertions


