//=========================================================================
//
//	FILE:				CP_M_Persist.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for objects that need to be able to be
//						read and written to and from an XML file.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_M_Persist
#define _H_CP_M_Persist
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueList.h"
#include "CP_Object.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_View;

/*!
*	@class	CP_M_Persist
*	@brief	Mix-in class for objects that need to be able to be read and written to and from an XML file.
*/
class CP_EXPORT CP_M_Persist : public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_M_Persist();
	//@}
			
	//! @name Initialization
	//@{
			void						InitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
			
										/*!
										*	Called when we have constructed our persist objects and are
										*	now giving each one a chance to finish up. Calls OnFinishPersist
										*	to do the actual work.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						FinishPersist()								{ OnFinishPersist(); }
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_Persist();
	//@}

	//! @name Initialization
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );

										/*!
										*	Override this do the actual finishing up of
										*	initializing the persist object. This is called after
										*	all of the other objects have been created. 
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						OnFinishPersist()							{}
	//@}

private:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_Persist

