//=========================================================================
//
//	FILE:				CP_M_PlugIn_Host.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for applications that want to provivide
//						ICP_PlugIn hosting services.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_M_PlugIn_Host
#define _H_CP_M_PlugIn_Host
#ifndef __BUILD_LINUX__
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

#if ! TARGET_OS_LINUX
	typedef std::vector<CP_FileSystemSpec, std::allocator<CP_FileSystemSpec> >						CP_FileSystemSpecs_Array;
	typedef std::vector<CP_FileSystemSpec, std::allocator<CP_FileSystemSpec> >::iterator			CP_FileSystemSpecs_Iterator;
	typedef std::vector<CP_FileSystemSpec, std::allocator<CP_FileSystemSpec> >::reverse_iterator	CP_FileSystemSpecs_ReverseIterator;
#else
	typedef std::vector<CP_FileSystemSpec >															CP_FileSystemSpecs_Array;
	typedef std::vector<CP_FileSystemSpec >::iterator												CP_FileSystemSpecs_Iterator;
	typedef std::vector<CP_FileSystemSpec >::reverse_iterator										CP_FileSystemSpecs_ReverseIterator;
#endif

	// Forward class declarations
class ICP_PlugIn;

	// Function proc ptr to load our ICP_PlugIn from the plug in library.
typedef void (*LoadPlugInProc)( ICP_PlugIn **inOutPlugIn );

class CP_M_PlugIn_Host
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_M_PlugIn_Host() throw();
	//@}

	//! @name PlugIns List
	//@{
			void						LoadPlugIns( const CP_FileSystemSpec& inFileSpec );						

			void						AddPlugIn( const CP_FileSystemSpec& inFileSpec );
			SInt16						GetNumberPlugIns();
			CP_FileSystemSpec			GetNthPlugIn( SInt16 inindex);
	//@}
			
protected:
	//! @name Constructor / Destructor
	//@{
	//! Contructor.
										CP_M_PlugIn_Host();
	//@}

	//! @name Loading
	//@{
			ICP_PlugIn					*LoadPlugIn( const CP_String& inPlugInName );
			ICP_PlugIn					*LoadPlugIn( const CP_FileSystemSpec& inPlugInSpec );
	//@}

private:

	// Class data
protected:
	CP_FileSystemSpecs_Array			*fPlugIns;		// list of all plugins
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_PlugIn_Host

