//=========================================================================
//
//	FILE:				CP_new.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Operator new and delete memory allocators.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 174 $
//						Latest Checkout:  $Date: 2005-05-16 14:29:14 -0700 (Mon, 16 May 2005) $
//
//=========================================================================
#ifndef _H_CP_New
#define _H_CP_New
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

	// Standard Library
#if TARGET_OS_LINUX
	#include <new>
	#include <memory>
	#include <stdexcept>
#else
	#include <new>
	#include <stdexcept>
#endif

#if TARGET_CC_MSVC
	#pragma warning(disable : 4290)		// operator new
#endif

#if ! defined(__CP_FRAMEWORK_DLL__) || ! defined(__CP_FRAMEWORK_DLL_APP__)
	#if CP_Debug_UseCustomOperatorNew
		#if CP_Debug_RecordMemoryAllocations
			#if TARGET_RT_MAC_MACHO
				void	*operator new( size_t size, const char *inFunction, const char *inFile, int inLine ) throw(std::bad_alloc);
				void	*operator new( size_t inSize, const std::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
				void 	*operator new[]( size_t inSize, const char *inFunction, const char *inFile, int inLine ) throw(std::bad_alloc);
				void 	*operator new[]( size_t inSize, const std::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
			#else
				#if ! TARGET_CC_MSVC
					void	*operator new( size_t size, const char *inFunction, const char *inFile, int inLine ) throw(std::bad_alloc);
					void	*operator new( size_t inSize, const std::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
					void 	*operator new[]( size_t inSize, const char *inFunction, const char *inFile, int inLine ) throw(std::bad_alloc);
					void 	*operator new[]( size_t inSize, const std::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
				#else
					void	*operator new( size_t size, const char *inFunction, const char *inFile, int inLine );
					void	*operator new( size_t inSize, const CP_STD::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
					void 	*operator new[]( size_t inSize, const char *inFunction, const char *inFile, int inLine ) throw(std::bad_alloc);
					void 	*operator new[]( size_t inSize, const CP_STD::nothrow_t&, const char *inFunction, const char *inFile, int inLine ) throw();
					void 	*operator new[]( size_t inSize ) throw(std::bad_alloc);
				#endif	// ! TARGET_CC_MSVC
			#endif	// TARGET_RT_MAC_MACHO
		#else
			#ifdef TARGET_RT_MAC_MACHO
					void	*operator new( size_t size ) throw(std::bad_alloc);
					void	*operator new( size_t inSize, const std::nothrow_t& ) throw();
					void 	*operator new[]( size_t inSize ) throw(std::bad_alloc);
					void 	*operator new[]( size_t inSize, const std::nothrow_t& ) throw();
			#else
				#if ! TARGET_CC_MSVC
					void	*operator new( size_t size ) throw(std::bad_alloc);
					void	*operator new( size_t inSize, const std::nothrow_t& ) throw();
					void 	*operator new[]( size_t inSize ) throw(std::bad_alloc);
					void 	*operator new[]( size_t inSize, const std::nothrow_t& ) throw();
				#else
					void	*operator new( size_t size ) throw(CP_STD::bad_alloc);
					void	*operator new( size_t inSize, const CP_STD::nothrow_t& ) throw();
					void 	*operator new[]( size_t inSize ) throw(CP_STD::bad_alloc);
					void 	*operator new[]( size_t inSize, const CP_STD::nothrow_t& ) throw();
				#endif	// ! TARGET_CC_MSVC
			#endif	// TARGET_RT_MAC_MACHO
		#endif	// CP_Debug_RecordMemoryAllocations
		
		void 	operator delete( void *obj ) throw();
		void 	operator delete( void *obj, const char *, int ) throw();
		void 	operator delete( void *obj, const std::nothrow_t&, const char *, int ) throw();
		void 	operator delete[]( void *inObj ) throw();
		void 	operator delete[]( void *obj, const char *, int ) throw();
		void 	operator delete[]( void *obj, const std::nothrow_t&, const char *, int ) throw();

		void	*CPLAT_OperatorNew( unsigned long inSize, bool isArray, const char *inFunction, const char *inFile, int inLine );
		void 	CPLAT_OperatorDelete( void *inObj, bool isArray );
	#endif	// CP_Debug_UseCustomOperatorNew
#endif	// ! defined(__CP_FRAMEWORK_DLL__) || ! defined(__CP_FRAMEWORK_DLL_APP__)

#if CP_Debug_UseCustomOperatorNew
	#if CP_Debug_RecordMemoryAllocations
		#define CP_new new(__FUNCTION__, __FILE__, __LINE__)
	#else
		#define CP_new new
	#endif	// __CP_Debug_RecordMemoryAllocations__
#else
	#define CP_new	new
#endif	// CP_Debug_UseCustomOperatorNew

#endif	// _H_CP_New
