//=========================================================================
//
//	FILE:				CP_Preferences.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage preference data, using key/value paring.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_Preferences
#define _H_CP_Preferences
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_UUID;
class CP_DateTime;

#if TARGET_OS_MAC
	#define kCP_CurrentApplicationID				kCFPreferencesCurrentApplication
#endif

#if TARGET_OS_WIN32 || TARGET_OS_LINUX
	#define kCP_CurrentApplicationID				""
#endif

/*!
*	@class		CP_Preferences
*	@brief		A class to manage preference data, using key/value paring.
*/
class CP_EXPORT CP_Preferences
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Preferences( const CP_String& inApplicationID = kCP_CurrentApplicationID, bool inWriteToAnyUserDomain=false );
										CP_Preferences( const CP_Preferences& inPreferences );
	//! Destructor.
	virtual								~CP_Preferences();
	//@}

	//! @name Getters
	//@{
			bool						GetValue(
												const CP_String& inKey,
												bool *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												SInt16 *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												SInt32 *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												float *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												double *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_Point *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_Size *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_Rect *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_RGBColor *inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_String& inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												char *inOutValue,
												SInt32 inMaxLength );
			bool						GetValue(
												const CP_String& inKey,
												CP_FileSystemSpec& inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_UUID& inOutValue );
			bool						GetValue(
												const CP_String& inKey,
												CP_DateTime& inOutValue );
	//@}
			
	//! @name Operators
	//@{
			CP_Preferences&				operator =( const CP_Preferences& inPreferences );
	//@}

	//! @name Setters
	//@{
			bool						SetValue(
												const CP_String& inKey,
												bool inOutValue );
			bool						SetValue(
												const CP_String& inKey,
												SInt16 inOutValue );
			bool						SetValue(
												const CP_String& inKey,
												SInt32 inOutValue );
			bool						SetValue(
											const CP_String& inKey,
												float inOutValue );
			bool						SetValue(
											const CP_String& inKey,
												double inOutValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_String& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const char *inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_Point& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_Size& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_Rect& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_RGBColor& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_FileSystemSpec& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_UUID& inValue );
			bool						SetValue(
												const CP_String& inKey,
												const CP_DateTime& inValue );
	//@}

protected:

	// class data
protected:
	CP_String							fApplicationID;		/*! Application identifier used to store / find preference data. */
	#if TARGET_OS_MAC	
		bool							fWriteToAnyUserDomain; /*! If true, prefs will be written to "any user" domain, otherwise to "current user" domain */
	#endif	
	#if TARGET_OS_WIN32
		HKEY 							fHKeyRoot;			/*! Windows HKeyRoot for preference data. */
		CP_String						fKeyPath;			/*! Path to the registry key */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Preferences( HKEY inHKeyRoot );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Preferences

