//=========================================================================
//
//	FILE:				CP_Region.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage a platforms region data type.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 228 $
//						Latest Checkout:  $Date: 2005-07-04 16:35:04 -0700 (Mon, 04 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Region
#define _H_CP_Region
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_HRGN.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_Size.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Region
*	@brief		A class to manage a platforms region data type.
*/
class CP_EXPORT CP_Region
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Region();
										CP_Region(
												float inLeft,
												float inTop,
												float inRight,
												float inBottom );
										CP_Region(
												const CP_Point& inLocation,
												const CP_Size& inSize );
										CP_Region( const CP_Rect& inRect );
										CP_Region( const CP_Region& inRegion );
	//! Destructor.
	virtual								~CP_Region();
	//@}
				
	//! @name Comparers
	//@{
			bool						operator == ( const CP_Region& inRegion ) const;
			bool						operator != ( const CP_Region& inRegion ) const;
	//@}

	//! @name Getters
	//@{
			CP_Rect						Bounds() const;

		#if CP_UseDepreciated_10_4_APIs
										/*!
										*	Returns the platforms native region.
										*	
										*	@exception	none
										*	
										*	@return		OSRegion		The platforms native region data type.
										*/
			OSRegion					GetOSRegion() const									{ return( fRegion ); }
		#endif // CP_UseDepreciated_10_4_APIs
	//@}

	//! @name Math - Region input
	//@{
			CP_Region&					operator += ( const CP_Region& inRegion );
			CP_Region&					operator -= ( const CP_Region& inRegion );
			CP_Region					operator + ( const CP_Region& inRegion );
			CP_Region					operator - ( const CP_Region& inRegion );

			CP_Region&					operator = ( const CP_Region& inRegion );
	//@}

	//! @name Math - Point input
	//@{
			CP_Region					operator += ( CP_Point& inPoint );
			CP_Region&					operator -= ( const CP_Point& inPoint );
			CP_Region					operator + ( CP_Point& inPoint );
			CP_Region					operator - ( const CP_Point& inPoint );
	//@}

	//! @name Movement
	//@{
			void						Offset(
												float inHorAmt,
												float inVerAmt );
			void						Offset( const CP_Point& amt );
	//@}

	//! @name Setters
	//@{
			void						Set(
												float inLeft,
												float inTop,
												float inRight,
												float inBottom );
			void						Set(
												const CP_Point& inLocation,
												const CP_Size& inSize );
			void						Set( const CP_Rect& inRect );

			void						Intersect( const CP_Region& area );
			void						Union( CP_Region& area );
			void						Difference( CP_Region& inRegion );
	//@}

	//! @name Sizers
	//@{
			void						Inset(
												float horAmt,
												float verAmt );
			void						Inset( const CP_Point& amt );
	//@}

	//! @name Testers
	//@{
			bool						IsEmpty() const;
			bool						IsValid() const;

			bool						Contains( const CP_Rect& inRect );
			bool						Contains( const CP_Point& inPoint );
	//@}

	//! @name Utility
	//@{
			void						MakeEmpty();

			float						Width() const;
			float						Height() const;
	//@}

private:
	//! @name Empty Region
	//@{
			void						MakeEmptyPlatformRegion();

	// class data
protected:
	#if TARGET_OS_MAC
		#if CP_UseDepreciated_10_4_APIs
			OSRegion					fRegion;			/*! The OSs region object. */
		#else
			#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)
				HIShapeRef				fShapeRef;			/*! A MacOS HIShapeRef to replace regions on 10.2 and above. */
			
				RgnHandle				fTempRegion;		/*! Needed for APIs that still require regions */
			#else
				OSRegion				fRegion;			/*! The OSs region object. */
			#endif // defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)
		#endif
	#else
		OSRegion						fRegion;			/*! The OSs region object. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
										CP_Region( RgnHandle inRegion );
	//@}

	#if CP_UseDepreciated_10_4_APIs
		//! @name Converters
		//@{
										/*!
										*	operator RgnHandle. Converts and returns this object as a
										*	MacOS RgnHandle.
										*	
										*	@exception	none
										*	
										*	@return		RgnHandle		A MacOS RgnHandle for this region.
										*/
				operator				RgnHandle() const									{ return( fRegion ); }
			
										/*!
										*	Returns this object as a MacOS RgnHandle.
										*	
										*	@exception	none
										*	
										*	@return		RgnHandle		A MacOS RgnHandle for this region.
										*/
				RgnHandle				Get()												{ return( fRegion ); }
				void					Set( RgnHandle inRegion );
		//@}
	#else
				void					Set( RgnHandle inRegion );

				RgnHandle				GetAsQDRegion();

		#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)
				void					Set( HIShapeRef inShapeRef );
		#else
										/*!
										*	operator RgnHandle. Converts and returns this object as a
										*	MacOS RgnHandle.
										*	
										*	@exception	none
										*	
										*	@return		RgnHandle		A MacOS RgnHandle for this region.
										*/
				operator				RgnHandle() const									{ return( fRegion ); }
			
										/*!
										*	Returns this object as a MacOS RgnHandle.
										*	
										*	@exception	none
										*	
										*	@return		RgnHandle		A MacOS RgnHandle for this region.
										*/
				RgnHandle				Get()												{ return( fRegion ); }
		#endif	//  defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)
	#endif
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
										CP_Region( HRGN inRegion );
	//@}

	//! @name Converters
	//@{
										/*!
										*	operator hrgn. Converts and returns this object as a
										*	Windows HRGN.
										*	
										*	@exception	none
										*	
										*	@return		HRGN		A Windows HRGN for this region.
										*/
			operator					HRGN() const										{ return( fRegion ); }

			CP_Region&					operator = ( CP_HRGN& inRegion );
				
			HRGN						Get();
			void						Set( HRGN inRegion );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Region

