//=========================================================================
//
//	FILE:				CP_FileFilter.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class used with the file iterator to filter out the
//						types of files listed.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_FileFilter
#define _H_CP_FileFilter
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_SignalSlot.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

typedef CP_SigSlot::CP_Signal2<bool&, const CP_FileSystemSpec&>				CP_FileFilterSignal;

#if TARGET_OS_LINUX
	typedef std::vector<UInt32>																CP_FileTypes_Array;
	typedef std::vector<UInt32>::iterator													CP_FileTypes_Iterator;
	typedef std::vector<UInt32>::reverse_iterator											CP_FileTypes_ReverseIterator;
#else
	typedef std::vector<UInt32, std::allocator<UInt32> >									CP_FileTypes_Array;
	typedef std::vector<UInt32, std::allocator<UInt32> >::iterator							CP_FileTypes_Iterator;
	typedef std::vector<UInt32, std::allocator<UInt32> >::reverse_iterator					CP_FileTypes_ReverseIterator;
#endif

/*!
*	@class		CP_FileFilter
*	@brief		Class used with the file iterator to filter out the types of files listed.
*
*				This class is used with the CP_FileIterator objects to provide finer
*				control of the objects returned by the file iterator.
*/
class CP_EXPORT CP_FileFilter : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileFilter();
										CP_FileFilter( CP_FileFilterSignal *inFilterSignal );
										CP_FileFilter(
												UInt32 inNumberTypes,
												UInt32 inFileTypes[] );
	//! Destructor.
	virtual								~CP_FileFilter() throw();
	//@}
			
	//! @name Filter Signal
	//@{
			CP_FileFilterSignal			*GetFilterSignal()									{ return( fFilterSignal ); }
			void						SetFilterSignal( CP_FileFilterSignal *inSignal )	{ fFilterSignal = inSignal; }
	//@}

	virtual	bool						IsAcceptable( const CP_FileSystemSpec& inSpec );
	
protected:

	// class data
protected:
	CP_FileFilterSignal					*fFilterSignal;
	
	CP_FileTypes_Array					fFileTypes;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileFilter

