//=========================================================================
//
//	FILE:				CP_Window_Imp_Gtk.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Linux / Gtk implemention of the CP_Window class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 159 $
//						Latest Checkout:  $Date: 2005-04-27 15:56:44 -0700 (Wed, 27 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_Window_Imp_Gtk
#define _H_CP_Window_Imp_Gtk
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_EventHandler.h"
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_OSStatus.h"
#include "CP_String.h"
#include "CP_View.h"
#include "CP_WindowTypes.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_Window_Imp : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Window_Imp();
	//! Destructor.
	virtual								~CP_Window_Imp() throw();
	//@}
				
	//! @name Appearance
	//@{
	virtual	void						Show();
	virtual	void						Hide();
	
			void						Select();
	//@}
			
	//! @name Commands
	//@{
	virtual	void						HandleCloseCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Events
	//@{
										/*!
										*	Returns the CP_EventHandler that handles events for this window.
										*	
										*	@exception	none
										*	
										*	@return		CP_EventHandler&		Returns a reference to our event handler.
										*/
	virtual	CP_EventHandler&			GetEventHandler()									{ return( fEventHandler ); }
	//@}

	//! @name Title
	//@{
			void						SetTitle( const CP_String& inTitle );
			CP_String					GetTitle() const;
	//@}
			
			GtkWidget					*GetMenuBarBox()									{ return( fMenuBarBox ); }
    		GtkWidget					*GetContentAreaContainer()							{ return( fContentAreaContainer ); }
    		void						SetContentAreaContainer( GtkWidget *inContainer)	{ fContentAreaContainer = inContainer; }

protected:
	//! @name Getters
	//@{
			OSWindow					GetImpPlatformWindow() const						{ return( fOSWindow ); }
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
	//@}

	//! @name Events
	//@{
	virtual void						OnClose();
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the MacOS WindowRef that this window object is using for its platform window.
										*
										*	@param		inWindow			The OS window to use.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetImpPlatformWindow( OSWindow inWindow )			{ fOSWindow = inWindow; }
	//@}

	// class data
protected:
	CP_WindowTypeBase					fWindowType;			/*! CP_WindowType object that describes the appearance of this window. */
	
	CP_Size								fMinSize;				/*! Minimum size a window can shrink to. */
	CP_Size								fMaxSize;				/*! Maximum size a window can grow to. */

private:
	OSWindow							fOSWindow;				/*! Platforms window reference of the window. */
	CP_EventHandler						fEventHandler;			/*! CPLAT event handler object that handles all window events. */

    GtkWidget							*fMenuBarBox;
    GtkWidget							*fContentAreaContainer;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_LINUX

#endif	// _H_CP_Window_Imp_Gtk

