//=========================================================================
//
//	FILE:				CP_View_Imp_HITextView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			DESCRIBE CLASS HERE
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 62 $
//						Latest Checkout:  $Date: 2005-01-16 13:38:46 -0800 (Sun, 16 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_View_Imp_HITextView
#define _H_CP_View_Imp_HITextView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && CPLAT_BUILD_OSX_10_3_OR_LATER

	// CPLAT_Framework
#include "CP_KeyInputFilters.h"
#include "CP_MLTESupport.h"
#include "CP_Object.h"
#include "CP_View_Imp_HIView.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_View_Imp_HITextView : public CP_View_Imp_HIView, public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View_Imp_HITextView(
												CP_View_Imp *inOwnerViewImp,
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	//! Destructor.
	virtual								~CP_View_Imp_HITextView() throw();
	//@}

	//! @name Command Handling
	//@{
			void						HandleCommandStatus(
												CP_CommandStatus& inCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommandNo );
										
			void						HandleCutCommand( OSStatus& inOutStatus );
			void						HandleCopyCommand( OSStatus& inOutStatus );
			void						HandleClearCommand( OSStatus& inOutStatus );
			void						HandlePasteCommand( OSStatus& inOutStatus );
			void						HandleSelectAllCommand( OSStatus& inOutStatus );
			void						HandleUndoCommand( OSStatus& inOutStatus );
			void						HandleRedoCommand( OSStatus& inOutStatus );
	//@}

	//! @name Drawing
	//@{
			void						HandleDraw( ControlRef inControl );
	//@}

	//! @name Events
	//@{
			OSStatus					HandleMouseDown( CP_Event_Base& inEvent );
			OSStatus					HandleCarbonEvent(
												EventHandlerCallRef inHandler,
												EventRef inEvent );
	//@}

	//! @name Files
	//@{
			OSStatus					ReadTextFromFile( const CP_FileSystemSpec& inFileSpec );
	//@}

	//! @name Key Filter
	//@{
			void						AttachFilter( CP_KeyInputFilter *inFilter );
			CP_KeyInputFilter			*GetFilter();
	//@}

	//! @name Getters
	//@{
			CP_String					GetText( char inLineEnd = CP_LF );
			UInt32						GetTextLength();
	virtual	CP_Rect						GetPortBounds();
	//@}

	//! @name Positioning
	//@{
	virtual	void						SetPortBounds( const CP_Rect& inBounds );
	//@}

	//! @name Printing
	//@{
			void						Print();
	//@}

	//! @name Selection
	//@{
			void						GetSelection(
												UInt32 *inStartSelection,
												UInt32 *inEndSelection );
			void						SetSelection(
												UInt32 inStartSelection,
												UInt32 inEndSelection );
			bool						HasSelection();
	//@}

	//! @name Setters
	//@{
			void						SetText( const CP_String& inText );
			void						SetText( CP_InStream *inStream );
			void						InsertText( const CP_String& inText );
			void						InsertText(
												const char *inTextPtr,
												SInt32 inTextLength );
			void						SetFontSpec( CP_FontSpec& inFontSpec );
	//@}

protected:
	//! @name Events
	//@{
	virtual	bool						HandleFieldExitChar( char inChar );
	//@}

private:
	//! @name Initialization
	//@{
			void						Init(
												CP_View *inSuperview,
												const CP_Point& inLocation,
												const CP_Size& inContentSize,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false );
	//@}

	//! @name Events
	//@{
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
			void						CommandProcess( CP_Event_Base& inEvent );

			void						HandleTextInputEvents( CP_Event_Base& inEvent );

	static pascal ::OSStatus			CarbonEventHandler(
												EventHandlerCallRef inHandler,
												EventRef inEvent,
												void *inUserData );
	//@}

	//! @name Misc
	//@{
			void 						TextChanged();
	//@}

	// class data
protected:
	CP_MLTESupport						fMLTESupport;
	HIViewRef							fScrollView;

	EventHandlerUPP						fHandlerUPP;
	EventHandlerRef						fHandlerRef;

	CP_KeyInputFilter					*fKeyFilter;			// object used to filter our keystrokes

		// signals
	CP_DataValueSignal					fFieldExitSignal;		/*! Signal sent when the user enters a field exit key. */

	CP_Point							fTextViewLocation;
};
	
CPLAT_End_Namespace_CPLAT

#endif	// TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && CPLAT_BUILD_OSX_10_3_OR_LATER

#endif	// _H_CP_View_Imp_HITextView

