//=========================================================================
//
//	FILE:				CP_PopupMenu.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms popup menu control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 202 $
//						Latest Checkout:  $Date: 2005-06-03 14:26:44 -0700 (Fri, 03 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_PopupMenu
#define _H_CP_PopupMenu
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Menu.h"
#include "CP_PlatformControl.h"
#include "CP_StaticText.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_PopupMenu;

#if TARGET_OS_LINUX
		// structure to hold info about a single menu item in our menu.
		// this structure is passed to the menu item callback routine when a user
		// selects a menu item
	struct CP_S_PopupMenuItem_Linux
	{
		CP_PopupMenu	*fOwnerMenu;			// pointer to ourselves -- CP_Menu we are a menu item in
		SInt16			fMenuItemIndex;
		GtkWidget		*fMenuItemWidget;		// Gtk menu item widget
	};
	
	typedef std::vector<CP_S_PopupMenuItem_Linux * >										CP_PopupMenuItems_Array;
	typedef std::vector<CP_S_PopupMenuItem_Linux * >::iterator								CP_PopupMenuItems_Iterator;
	typedef std::vector<CP_S_PopupMenuItem_Linux * >::reverse_iterator						CP_PopupMenuItems_ReverseIterator;
#endif	// TARGET_OS_LINUX

/*!
*	@class		CP_PopupMenu
*	@brief		Implementation of a platforms popup menu control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_PopupMenu : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_PopupMenu();
										CP_PopupMenu(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												CP_String& inTitle,
												SInt16 inMenuID,
												SInt16 inTitleWidth = -1 );
	//! Destructor.
	virtual								~CP_PopupMenu() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Setters
	//@{
				void					SetMenu( SInt16 inMenuID );
				void					SetMenu( CP_Menu *inMenu );
	//@}

protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	// class data
protected:
	CP_Menu								*fMenu;
	
	#if TARGET_OS_WIN32
									/*! Helper object to display the title for Windows and Linux. */
		CP_StaticText					*fTitlePane;

		CP_EventHandler					fEventHandler;		/*! Class that handles all of the events for this windnow. */
		WNDPROC							fnOldWndProc;
	#endif

	#if TARGET_OS_LINUX
										/*! List of structures we maintain about each item in the menu, used to map between CPLAT
										information for a menu, and the GtkWidget */
		CP_PopupMenuItems_Array			*fMenuItemStructs_Linux;
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Carbon Events
	//@{
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
			void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												CP_String& inTitle,
												SInt16 inTitleWidth = -1 );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	virtual	void						SetValue( SInt32 inValue );

			LRESULT						HandleWinProcMessage(
												HWND inHwnd,
												UINT inMsg,
												WPARAM inWParam,
												LPARAM inLParam );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												CP_String& inTitle,
												SInt16 inTitleWidth = -1 );
	//@}

	//! @name Menu
	//@{
			void						LoadMenuItems( OSMenu inOSMenu );
			void						CommandProcess( CP_Event_Base& inEvent );
	//@}
#endif

#if TARGET_OS_LINUX
public:
			void						HandlePopupSelection( CP_S_PopupMenuItem_Linux *inSelection );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												CP_String& inTitle,
												SInt16 inTitleWidth = -1 );
	//@}

			void						AddMenuItem_Linux( CP_S_PopupMenuItem_Linux *inMenuItem );

	//! @name Menu
	//@{
			void						LoadMenuItems( OSMenu inOSMenu );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PopupMenu
