//=========================================================================
//
//	FILE:				CP_WindowTypes.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes for defining the behavior of windows in the
//						CPLAT framework. The CP_Window object uses an instance
//						of one of these classes to implement the behavior of
//						a single type of window, such as Documents, Floating,
//						Dialog, Alert, etc.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 53 $
//						Latest Checkout:  $Date: 2005-01-12 14:31:22 -0800 (Wed, 12 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_WindowTypes
#define _H_CP_WindowTypes
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_OSTypes.h"
#include "CP_RGBColor.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
		// Window attributes to describe the various features available for
		// CPLAT windows.
	enum
	{
		kCP_E_WindowNoAttribute = kWindowNoAttributes,
		kCP_E_WindowCloseBoxAttribute = kWindowCloseBoxAttribute,
		kCP_E_WindowHorizontalZoomAttribute = kWindowHorizontalZoomAttribute,
		kCP_E_WindowVerticalZoomAttribute = kWindowVerticalZoomAttribute,
		kCP_E_WindowFullZoomAttribute = kWindowFullZoomAttribute,

		kCP_E_WindowCollapseBoxAttribute = kWindowCollapseBoxAttribute ,

		kCP_E_WindowResizableAttribute = kWindowResizableAttribute,

		kCP_E_WindowStandardHandlerAttribute = kWindowStandardHandlerAttribute,
		
		kCP_E_WindowStandardDocumentAttribute = kWindowStandardDocumentAttributes,

		#if UNIVERSAL_INTERFACES_VERSION >= 0x0400
			kCP_E_WindowMetalAttribute = kWindowMetalAttribute
		#else
			kCP_E_WindowMetalAttribute = (1L << 8)
		#endif
	};
	
	enum
	{
		kCP_E_WindowClass_Alert = kAlertWindowClass,
		kCP_E_WindowClass_MovableAlert = kMovableAlertWindowClass,
		kCP_E_WindowClass_Modal = kModalWindowClass,
		kCP_E_WindowClass_MoveableModal = kMovableModalWindowClass,
		kCP_E_WindowClass_Floating = kFloatingWindowClass,
		kCP_E_WindowClass_Document = kDocumentWindowClass,
		kCP_E_WindowClass_Utility = kUtilityWindowClass,
		kCP_E_WindowClass_Help = kHelpWindowClass,
		kCP_E_WindowClass_Sheet = kSheetWindowClass,
		kCP_E_WindowClass_Toolbar = kToolbarWindowClass,
		kCP_E_WindowClass_Plain = kPlainWindowClass,
		kCP_E_WindowClass_Overlay = kOverlayWindowClass,
		kCP_E_WindowClass_SheetAlert = kSheetAlertWindowClass,
		kCP_E_WindowClass_AltPlain = kAltPlainWindowClass,
		#if UNIVERSAL_INTERFACES_VERSION >= 0x0400
			kCP_E_WindowClass_Drawer = kDrawerWindowClass
		#else
			kCP_E_WindowClass_Drawer = 20	//kDrawerWindowClass		// Not defined in early headers
		#endif
	};
#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32
		// Window attributes to describe the various features available for
		// CPLAT windows. 
	enum
	{
		kCP_E_WindowNoAttribute = 0L,
		kCP_E_WindowCloseBoxAttribute = (1L << 0),
		kCP_E_WindowHorizontalZoomAttribute = WS_MAXIMIZEBOX | WS_SYSMENU,
		kCP_E_WindowVerticalZoomAttribute = WS_MAXIMIZEBOX | WS_SYSMENU,
		kCP_E_WindowFullZoomAttribute  = (kCP_E_WindowVerticalZoomAttribute | kCP_E_WindowHorizontalZoomAttribute),

		kCP_E_WindowCollapseBoxAttribute = WS_MINIMIZEBOX,
		kCP_E_WindowResizableAttribute = kCP_E_WindowHorizontalZoomAttribute | kCP_E_WindowVerticalZoomAttribute,

		kCP_E_WindowStandardHandlerAttribute = WS_OVERLAPPED | WS_THICKFRAME | WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU,
		
		kCP_E_WindowStandardDocumentAttribute = (kCP_E_WindowCloseBoxAttribute | kCP_E_WindowFullZoomAttribute | kCP_E_WindowCollapseBoxAttribute | kCP_E_WindowResizableAttribute ),

		kCP_E_WindowMetalAttribute = (1L << 8)
	};

	enum
	{
		kCP_E_WindowClass_Alert = 1,
		kCP_E_WindowClass_MovableAlert = 2,
		kCP_E_WindowClass_Modal = 3,
		kCP_E_WindowClass_MoveableModal = 4,
		kCP_E_WindowClass_Floating = 5,
		kCP_E_WindowClass_Document = 6,
		kCP_E_WindowClass_Utility = 8,
		kCP_E_WindowClass_Help = 10,
		kCP_E_WindowClass_Sheet = 11,
		kCP_E_WindowClass_Toolbar = 12,
		kCP_E_WindowClass_Plain = 13,
		kCP_E_WindowClass_Overlay = 14,
		kCP_E_WindowClass_SheetAlert = 15,
		kCP_E_WindowClass_AltPlain = 16,
		kCP_E_WindowClass_Drawer = 20
	};
#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX
		// Window attributes to describe the various features available for
		// CPLAT windows.
	enum
	{
		kCP_E_WindowNoAttribute = 0L,
		kCP_E_WindowCloseBoxAttribute = (1L << 0),
		kCP_E_WindowHorizontalZoomAttribute = (1L << 1),
		kCP_E_WindowVerticalZoomAttribute = (1L << 2),
		kCP_E_WindowFullZoomAttribute  = (kCP_E_WindowVerticalZoomAttribute | kCP_E_WindowHorizontalZoomAttribute),

		kCP_E_WindowCollapseBoxAttribute = (1L << 3),
		kCP_E_WindowResizableAttribute = (1L << 4),

		kCP_E_WindowStandardHandlerAttribute = (1L << 25),
		
		kCP_E_WindowStandardDocumentAttribute = (kCP_E_WindowCloseBoxAttribute | kCP_E_WindowFullZoomAttribute | kCP_E_WindowCollapseBoxAttribute | kCP_E_WindowResizableAttribute ),

		kCP_E_WindowMetalAttribute = (1L << 8)
	};

	enum
	{
		kCP_E_WindowClass_Alert = 1,
		kCP_E_WindowClass_MovableAlert = 2,
		kCP_E_WindowClass_Modal = 3,
		kCP_E_WindowClass_MoveableModal = 4,
		kCP_E_WindowClass_Floating = 5,
		kCP_E_WindowClass_Document = 6,
		kCP_E_WindowClass_Utility = 8,
		kCP_E_WindowClass_Help = 10,
		kCP_E_WindowClass_Sheet = 11,
		kCP_E_WindowClass_Toolbar = 12,
		kCP_E_WindowClass_Plain = 13,
		kCP_E_WindowClass_Overlay = 14,
		kCP_E_WindowClass_SheetAlert = 15,
		kCP_E_WindowClass_AltPlain = 16,
		kCP_E_WindowClass_Drawer = 20
	};
#endif	// TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_WindowTypeBase ==
#pragma mark -
#endif

/*!
*	@class		CP_WindowTypeBase
*	@brief		Base class for a window type definition.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_WindowTypeBase();
										CP_WindowTypeBase( const CP_WindowTypeBase& inWindowBase );
	//! Destructor.
	virtual								~CP_WindowTypeBase() throw();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return the class this window belongs to (MacOS).
										*	
										*	@return	UInt32 const		Class the window belongs to.
										*/
			UInt32						GetWindowClass() const											{ return( fWindowClass ); }

										/*!
										*	Return the attributes of this window (MacOS).
										*	
										*	@return	UInt32 const		Attributes of the window.
										*/
			UInt32						GetWindowAttributes() const										{ return( fWindowAttributes ); }

										/*!
										*	Return the window flags (Windows).
										*	
										*	@return	UInt32 const		Window flags.
										*/
			UInt32						GetWindowFlags() const											{ return( fWindowFlags ); }

										/*!
										*	Return the extended window flags (Windows).
										*	
										*	@return	UInt32 const		Extended window flags.
										*/
			UInt32						GetExtendedWindowFlags() const									{ return( fExtendedWindowFlags ); }
	//@}

	//! @name Setters
	//@{
			void						SetWindowClass( const UInt32 inWindowClass )					{ fWindowClass = inWindowClass; }
			void						SetWindowAttributes( const UInt32 inWindowAttributes )			{ fWindowAttributes = inWindowAttributes; }
			void						SetWindowFlags( const UInt32 inWindowFlags )					{ fWindowFlags = inWindowFlags; }
			void						SetExtendedWindowFlags( const UInt32 inExtendedWindowFlags )	{ fExtendedWindowFlags = inExtendedWindowFlags; }
	//@}

	//! @name Operators
	//@{
			CP_WindowTypeBase&			operator = ( const CP_WindowTypeBase& inWindowBase );
	//@}

protected:

	// class data
protected:
	UInt32								fWindowClass;			/*! Class that window belongs to.  (MacOS) */
	UInt32								fWindowAttributes;		/*! Attributes used to create the window.  (MacOS) */
	UInt32								fWindowFlags;			/*! Flags passed to the window.  (Windows) */
	UInt32								fExtendedWindowFlags;	/*! Extended flags passed to the window. (Windows) */
	
	#if TARGET_OS_MAC
		SInt16							fThemeBrush;
	#endif
	
	#if TARGET_OS_WIN32
		CP_RGBColor						fBackgroundColor;		/*! Color of the windows background. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Getters
	//@{
			SInt16						GetThemeBrush()	const										{ return( fThemeBrush ); }
	//@}

	//! @name Setters
	//@{
			void						SetThemeBrush( const SInt16 inThemeBrush )					{ fThemeBrush = inThemeBrush; }
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Getters
	//@{
	#if UNICODE
		virtual	CPUniChar				*GetWindowsWindowClass() const;
	#else
		virtual	char					*GetWindowsWindowClass() const;
	#endif

				CP_RGBColor				GetWindowBackgroundColor() const;
	//@}

				void					SetWindowBackgroundColor( const CP_RGBColor& inColor )		{ fBackgroundColor = inColor; }

protected:
	//! @name Registration
	//@{
				void					RegisterWindowClass();
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DocumentWindowType ==
#pragma mark -
#endif

/*!
*	@class		CP_DocumentWindowType
*	@brief		Window type definition for a document window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_DocumentWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DocumentWindowType(
												UInt32 inAttributes = kCP_E_WindowStandardDocumentAttribute | kCP_E_WindowStandardHandlerAttribute,
												bool inUseCompositing = true );
	//! Destructor.
	virtual								~CP_DocumentWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
	//! @name Getters
	//@{
	#if UNICODE
		virtual	CPUniChar				*GetWindowsWindowClass() const;
	#else
		virtual	char					*GetWindowsWindowClass() const;
	#endif
	//@}

protected:
	//! @name Registration
	//@{
	virtual	void						RegisterWindowClass();
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_AlertWindowType ==
#pragma mark -
#endif

/*!
*	@class		CP_AlertWindowType
*	@brief		Window type definition for an alert window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_AlertWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AlertWindowType();
	//! Destructor.
	virtual								~CP_AlertWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DialogWindowTypes ==
#pragma mark -
#endif

/*!
*	@class		CP_ModelessDialogWindowType
*	@brief		Window type definition for a modeless dialog window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_ModelessDialogWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ModelessDialogWindowType();
	//! Destructor.
	virtual								~CP_ModelessDialogWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_ModalDialogWindowType
*	@brief		Window type definition for a modal dialog window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_ModalDialogWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ModalDialogWindowType();
	//! Destructor.
	virtual								~CP_ModalDialogWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_FloatingWindowTypes ==
#pragma mark -
#endif

/*!
*	@class		CP_FloatDialogWindowType
*	@brief		Window type definition for a floating window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_FloatingWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FloatingWindowType();
	//! Destructor.
	virtual								~CP_FloatingWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_SheetWindowType ==
#pragma mark -
#endif

/*!
*	@class		CP_SheetWindowType
*	@brief		Window type definition for a floating window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_SheetWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SheetWindowType();
	//! Destructor.
	virtual								~CP_SheetWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DrawerWindowType ==
#pragma mark -
#endif

/*!
*	@class		CP_DrawerWindowType
*	@brief		Window type definition for a floating window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_DrawerWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DrawerWindowType();
	//! Destructor.
	virtual								~CP_DrawerWindowType() throw();
	//@}

protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_MDIClientWindowType ==
#pragma mark -
#endif

class CP_EXPORT CP_MDIClientWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MDIClientWindowType();
	//! Destructor.
	virtual								~CP_MDIClientWindowType() throw();
	//@}

	//! @name Getters
	//@{
	#if UNICODE
		virtual	CPUniChar				*GetWindowsWindowClass() const;
	#else
		virtual	char					*GetWindowsWindowClass() const;
	#endif
	//@}

protected:

	// class data
protected:
	
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_MDIFrameWindowType ==
#pragma mark -
#endif

class CP_EXPORT CP_MDIFrameWindowType : public CP_WindowTypeBase
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MDIFrameWindowType();
	//! Destructor.
	virtual								~CP_MDIFrameWindowType() throw();
	//@}

	//! @name Getters
	//@{
	#if UNICODE
		virtual	CPUniChar				*GetWindowsWindowClass() const;
	#else
		virtual	char					*GetWindowsWindowClass() const;
	#endif
	//@}

	//! @name Registration
	//@{
			void						RegisterWindowClass();
	//@}

protected:

	// class data
protected:
	
};

#endif	// TARGET_OS_WIN32

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_WindowTypes

