//=========================================================================
//
//	FILE:				CP_Application_Imp_Win.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Windows implementation of the CP_Application class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 274 $
//						Latest Checkout:  $Date: 2005-09-09 22:32:33 -0700 (Fri, 09 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_Application_Imp_Win
#define _H_CP_Application_Imp_Win
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_Enums.h"
#include "CP_Event.h"
#include "CP_EventHandler.h"
#include "CP_M_CommandHandler.h"
#include "CP_M_DeferredTask.h"
#include "CP_OSStatus.h"
#include "CP_Object.h"
#include "CP_OSStatus.h"
#include "CP_ScriptableObject.h"
#include "CP_SignalSlot.h"
#include "CP_TimedTask.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// typedef 
#if TARGET_CC_CODEWARRIOR
	typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >				CP_M_DeferredTask_Array;
	typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >::iterator	CP_M_DeferredTask_Iterator;
#else
	typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >					CP_M_DeferredTask_Array;
	typedef std::vector<CP_M_DeferredTask *, std::allocator<CP_M_DeferredTask *> >::iterator			CP_M_DeferredTask_Iterator;
#endif

/*!
*	@class		CP_Application_Imp
*	@brief		Windows implementation of the CP_Application class.
*	
*				This version is based on a Windows event loop for all event handling.
*/
class CP_EXPORT CP_Application_Imp : public CP_ScriptableObject, public CP_M_CommandHandler, public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Application_Imp();
	//! Destructor.
	virtual								~CP_Application_Imp() throw();
	//@}
	
	//! @name Instance
	//@{
	static	CP_Application_Imp			*Instance();
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleQuitCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePreferencesCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name DeferredTasks
	//@{
			void						AddTask( CP_M_DeferredTask *inTask );
			void						RemoveTask( CP_M_DeferredTask *inTask );
			void						RemoveAllTasks();
			UInt32						GetNumberTasks();
			UInt32						GetTaskListIndex( CP_M_DeferredTask *inTask );

	virtual	void						DoDeferredTasks();			
	//@}

	//! @name Getters
	//@{
			CP_EventHandler&			GetEventHandler()									{ return( fEventHandler ); }
	//@}


	//! @name Application Status
	//@{
			CP_E_AppStatus				GetAppStatus();

										/*!
										*	Sets the status of our application from the passed in parameter.
										*	
										*	@param	inAppStatus			Flag to indicate the current condition of the application.
										*								These flags are defined in CP_Enums.h and indicate if the application
										*								is starting up, running, quitting, or quit.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetAppStatus( CP_E_AppStatus inAppStatus )						{ fAppStatus = inAppStatus; }
	//@}

	//! @name Events
	//@{
	virtual	void						Run();
	//@}

	//! @name Exceptions
	//@{
	virtual void 						OnUncaughtException( const std::exception& e );
	//@}

protected:
	//! @name Commands
	//@{
	virtual	void						HandleNewUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandleNewCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandleOpenUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandleOpenCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandlePageSetupUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandlePageSetupCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	virtual	void						HandlePrintUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& /*inOutWasHandled*/,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutCommandStatus.SetIsEnabled( false ); }
	virtual	void						HandlePrintCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& /*inCommandNo*/ )					{ inOutStatus = CP_noErr; }
	//@}

	//! @name Documents
	//@{
	virtual	void						RegisterDocuments();
	//@}

	//! @name Events
	//@{
	virtual	void						HandleOpenApplication()									{}
	virtual	void						HandleShowPreferences()									{}
	//@}
	
	//! @name Initialization
	//@{
	virtual	void						PrepareToRun();
	//@}

private:
	//! @name Initialization
	//@{
			void						Init();
			void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}
			
	//! @name Commands
	//@{
			void						CommandProcess( CP_Event_Base& inEvent );
			void						CommandUpdateStatus( CP_Event_Base& inEvent );

	virtual	bool						HandleQuitApplication()									{ return( false ); }
	//@}

	//! @name Events
	//@{
			void						RunOneEventLoop( CP_Event<kCP_WINDOW_EVENT, 0>& inOutEvent );
			bool						GetAnEvent( CP_Event<kCP_WINDOW_EVENT, 0>& inOutEvent );
			void						DispatchEvent( CP_Event<kCP_WINDOW_EVENT, 0>& inEvent );
	//@}

	//! @name Slots
	//@{
			void						HandleDeferredTasksTimer(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOSStatus );
	//@}

	// class data
protected:
	static CP_Application_Imp			*fsInstance;		/*! Single instance of this class. */

	CP_E_AppStatus						fAppStatus;			/*! Current state of the application, startup, running, quitting or quit. */
	CP_EventHandler						fEventHandler;		/*! Event handling object for this class */

	CP_M_DeferredTask_Array				*fDeferredTasks;	/*! Lists of tasks to be handled at idle time. */
	CP_TimedTask						*fDeferredTasksTimer;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32

#endif	// _H_CP_Application_Imp_Win

