//=========================================================================
//
//	FILE:				CP_MDIMainWindowManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for Windows only that handle the MDI
//						(Multiple Document Inferface) main frame window.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_MDIMainWindowManager
#define _H_CP_MDIMainWindowManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_MenuBar.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_WIN32

/*!
*	@class		CP_MDIMainWindowManager
*	@brief		A class for Windows only that handles the MDI (Multiple Document Inferface) main frame window.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_MDIMainWindowManager
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MDIMainWindowManager( const CP_String& inTitle );
	//! Destructor.
	virtual								~CP_MDIMainWindowManager() throw();
	//@}

	//! @name Instance
	//@{
	static 	CP_MDIMainWindowManager		*Instance();
	//@}

			void						CreateMDIClientWindow( OSWindow inMDIFrameWindow );

	//! @name Getters
	//@{
			CP_Window					*GetMDIClientWindow()								{ return( fsMDIClientWin ); }
			CP_Window					*GetMDIFrameWindow()								{ return( fsMDIFrameWin ); }
	//@}

	//! @name Setters
	//@{
			void						SetMenuBar( CP_MenuBar *inMenuBar );
	//@}

private:
	//! @name Registration
	//@{
			void						RegisterMDIChildWindowClass();
	//@}

	// class data
protected:
	static CP_MDIMainWindowManager		*fsInstance;		/*! The single instance of this class.*/
	
	CP_Window							*fsMDIClientWin;	/*! Windows client window.*/
	CP_Window							*fsMDIFrameWin;		/*! Windows frame window.*/
};

#if UNICODE
	extern CPUniChar	szMDIChildClass[];
	extern CPUniChar	szMDIFrameClass[];
#else
	extern char			szMDIChildClass[];
	extern char			szMDIFrameClass[];
#endif

#endif	//TARGET_OS_WIN32

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MDIMainWindowManager
